﻿//
// Parago Media GmbH & Co. KG, Jürgen Bäurle (jbaurle@parago.de)
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
//

using System;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;
using System.Security;
using Microsoft.BusinessData.Infrastructure.SecureStore;
using Microsoft.Office.SecureStoreService.Server;
using Microsoft.SharePoint;

namespace SP2010SecureStoreSAP
{
	internal class SecureStore
	{
		public string ApplicationId { get; private set; }

		public SecureStore(string applicationId)
		{
			if(string.IsNullOrEmpty(applicationId))
				throw new ArgumentNullException("applicationId");
			if(!IsApplicationValid(applicationId))
				throw new ArgumentException(string.Format("Target application with ID '{0}' is not defined.", applicationId));

			ApplicationId = applicationId;
		}

		public SecureStoreCredentials GetCurrentCredentials()
		{
			SecureStoreProvider provider = new SecureStoreProvider { Context = SPServiceContext.Current };
			string userName = string.Empty;
			string password = string.Empty;

			using(SecureStoreCredentialCollection data = provider.GetCredentials(ApplicationId))
			{
				foreach(ISecureStoreCredential c in data)
				{
					if(c != null)
					{
						if(c.CredentialType == SecureStoreCredentialType.UserName)
							userName = GetDecryptedCredentialString(c.Credential);
						else if(c.CredentialType == SecureStoreCredentialType.Password)
							password = GetDecryptedCredentialString(c.Credential);
					}
				}
			}

			if(string.IsNullOrEmpty(userName) || string.IsNullOrEmpty(password))
				throw new SecureStoreException("Credentials for the current Windows user are not valid or not defined.");

			return new SecureStoreCredentials(userName, password);
		}

		public static bool IsApplicationValid(string applicationId)
		{
			if(string.IsNullOrEmpty(applicationId))
				throw new ArgumentNullException("applicationId");

			SecureStoreProvider provider = new SecureStoreProvider { Context = SPServiceContext.Current };

			foreach(TargetApplication application in provider.GetTargetApplications())
			{
				if(application.ApplicationId == applicationId)
				{
					ReadOnlyCollection<ITargetApplicationField> fields = provider.GetTargetApplicationFields(applicationId);
					bool existsUserNameDefinition = false;
					bool existsPasswordDefinition = false;

					foreach(TargetApplicationField field in fields)
					{
						if(field.CredentialType == SecureStoreCredentialType.UserName)
							existsUserNameDefinition = true;
						else if(field.CredentialType == SecureStoreCredentialType.Password)
							existsPasswordDefinition = true;
					}

					if(existsUserNameDefinition && existsPasswordDefinition)
						return true;
				}
			}

			return false;
		}

		public static string GetDecryptedCredentialString(SecureString secureString)
		{
			IntPtr p = Marshal.SecureStringToBSTR(secureString);

			try
			{
				return Marshal.PtrToStringUni(p);
			}
			finally
			{
				if(p != IntPtr.Zero)
					Marshal.FreeBSTR(p);
			}
		}
	}
}
